
def get_total_seconds(timedelta):
    """
    Backported for Python < 2.7

    See http://docs.python.org/library/datetime.html.
    """
    return ((timedelta.microseconds + (
            timedelta.seconds +
            timedelta.days * 24 * 60 * 60
        ) * 10**6) / float(10**6))




import unittest
from porunga import get_manager
from porunga.commands.test import PorungaTestCommand


class TestManager(unittest.TestCase):

    def test_manager_has_proper_commands(self):
        manager = get_manager()
        commands = manager.get_commands()

        self.assertTrue('test' in commands)
        test_command = commands['test']
        self.assertTrue(isinstance(test_command, PorungaTestCommand))




"""
porunga is a test framework for simple algorithms
"""
import os
import sys


VERSION = (0, 9, 3)

__version__ = '.'.join((str(each) for each in VERSION[:4]))

def get_version():
    """
    Returns shorter version (digit parts only) as string.
    """
    version = '.'.join((str(each) for each in VERSION[:3]))
    if len(VERSION) > 3:
        version += VERSION[3]
    return version

def update_sys_path():
    dirpath = os.path.abspath(os.path.dirname(__file__))
    sys.path.insert(0, dirpath)

def get_manager():
    update_sys_path()
    from monolith.cli import ExecutionManager
    from porunga.utils.imports import import_class

    class PorungaManager(ExecutionManager):

        def get_commands_to_register(self):
            registry = {
                'test': 'porunga.commands.test.PorungaTestCommand',
            }
            commands = dict((name, import_class(path)) for name, path in
                registry.items())
            return commands

    return PorungaManager(['porunga'])


def main():
    manager = get_manager()
    manager.execute()




import os
import shutil
import sys
import tempfile
from argparse import Namespace
from porunga.commands.test import curdir
from porunga.commands.test import PorungaTestCommand
from porunga.utils.compat import unittest
from porunga.utils.paths import abspath
from mock import call
from mock import Mock
from mock import patch
from subprocess import PIPE
from termcolor import colored



class TestPorungaTestCommand(unittest.TestCase):

    def setUp(self):
        namespace = Namespace(quiet=True, all=False, lang='python', timeout=0)
        self.command = PorungaTestCommand()
        self.command.namespace = namespace
        self.command.namespace.verbose = True

    def test_get_binary(self):
        self.assertEqual(self.command.get_binary('foobar/', 'python'),
            'python foobar/foobar.py')

    def test_get_binary_in_subdirectory(self):
        self.assertEqual(self.command.get_binary('foobar/foo', 'python'),
            'python foobar/foo/foo.py')

    def test_get_binary_python(self):
        self.assertEqual(self.command.get_binary('foobar', 'python'),
            'python foobar/foobar.py')

    def test_get_binary_ruby(self):
        self.assertEqual(self.command.get_binary('foobar', 'ruby'),
            'ruby foobar/foobar.rb')

    def test_get_binary_java(self):
        self.command.compile = Mock()
        self.assertEqual(self.command.get_binary('foobar', 'java'),
            'java -cp foobar foobar')
        self.command.compile.assert_called_once_with('javac foobar/foobar.java')

    def test_get_binary_objc(self):
        self.command.compile = Mock()
        self.assertEqual(self.command.get_binary('foobar/', 'objc'),
            'foobar/foobar.m.out')
        self.command.compile.assert_called_once_with(
            'clang -ObjC -framework Foundation foobar/foobar.m -o foobar/foobar.m.out')

    def test_get_binary_c(self):
        self.command.compile = Mock()
        self.assertEqual(self.command.get_binary('foobar/', 'c'),
            'foobar/foobar.c.out')
        self.command.compile.assert_called_once_with(
            'gcc -O2 foobar/foobar.c -o foobar/foobar.c.out')

    def test_get_binary_cpp(self):
        self.command.compile = Mock()
        self.assertEqual(self.command.get_binary('foobar/', 'cpp'),
            'foobar/foobar.cpp.out')
        self.command.compile.assert_called_once_with(
            'g++ -O2 foobar/foobar.cpp -o foobar/foobar.cpp.out')

    # compile tests

    def test_compile_went_wrong(self):

        with patch('porunga.commands.test.Popen') as PopenMock:
            PopenMock.return_value = mock = Mock()
            mock.communicate = lambda: ('out', 'err')
            mock.returncode = 102
            with self.assertRaises(SystemExit) as err:
                self.command.compile('some prog')
                self.assertEqual(err.code, 102)

    def test_wrong_lang(self):
        with self.assertRaises(SystemExit):
            self.command.get_binary('foobar', 'foolang')

    def test_exit(self):
        with self.assertRaises(SystemExit) as err:
            self.command.exit('error message', 101)
            self.assertEqual(err.code, 101)

            self.command.namespace.quiet = False
            with patch.object(sys, 'stderr') as errmock:
                self.command.exit('message')
                self.assertEqual(errmock.write.call_args_list, [
                    call(colored('Error: ', 'red')),
                    call(colored('message', 'red')),
                    call('\n'),
                ])

    def test_get_test_cases_for_suffix(self):
        with patch('porunga.commands.test.glob') as mglob:
            mglob.return_value = ['foobar/test01.in', 'foobar/test02.in']
            cases = self.command.get_test_cases_for_suffix('foobar', 'in')
            self.assertItemsEqual(cases, [
                'foobar/test01.in',
                'foobar/test02.in',
            ])
            self.assertEqual(mglob.call_args, call('foobar/testdata/*.in'))

    def test_get_test_cases_for_suffix_all_flag(self):
        with patch('porunga.commands.test.walk') as walkmock:
            walkmock.return_value = [
                ('foobar/testdata', ['bigtests'], ['test01.in', 'test01.out', 'test02.in']),
                ('foobar/testdata/bigtests', [], ['test-big-01.in', 'test02.out']),
            ]
            self.command.namespace.all = True
            test_cases = self.command.get_test_cases_for_suffix('foobar/', 'in')

            walkmock.assert_called_once_with('foobar/testdata')
            self.assertEqual(test_cases, [
                'foobar/testdata/test01.in',
                'foobar/testdata/test02.in',
                'foobar/testdata/bigtests/test-big-01.in',
            ])

    def test_get_test_cases_with_case_option(self):
        self.command.namespace.case = '/foo/some.in'
        self.command.get_fout_name = Mock(return_value='/foo/some.out')
        cases = self.command.get_test_cases('foobar')
        self.assertEqual(cases, [('/foo/some.in', '/foo/some.out')])

    def test_get_test_cases(self):
        self.command.get_test_cases_for_suffix = Mock()
        self.command.get_test_cases_for_suffix.return_value = ['foo.in']
        self.command.get_fout_name = Mock(return_value='foo.out')
        result = self.command.get_test_cases('foobar')
        self.assertEqual(self.command.get_test_cases_for_suffix.call_args_list, [
            call('foobar', 'in'),
            call('foobar', 'IN'),
        ])
        self.assertEqual(result, [('foo.in', 'foo.out')])

    def test_get_test_cases_dupes_if_case_given(self):
        tempdir = tempfile.mkdtemp()
        testdir = abspath(tempdir, 'testdata')
        os.makedirs(testdir)
        fin = abspath(testdir, 'foo.in')
        fout = abspath(testdir, 'foo.out')
        open(fin, 'w')
        open(fout, 'w')
        self.command.namespace.case = fin
        cases = self.command.get_test_cases(tempdir)
        try:
            self.assertEqual(cases, [(fin, fout)])
        finally:
            shutil.rmtree(tempdir)

    def test_handle_label_error(self):
        with patch('porunga.commands.test.Popen') as PopenMock:
            self.command.get_test_cases = Mock(return_value=[
                ('test1.in', 'test1.out'), ('test2.in', 'test2.out')])
            mock = PopenMock.return_value = Mock()
            mock.communicate = lambda: ('out', 'err')
            mock.returncode = 0

            self.command.get_binary = Mock()
            self.command.info = Mock()
            self.command.error = Mock()

            self.command.handle_label('foobar', self.command.namespace)

            self.command.get_binary.assert_called_once_with('foobar', 'python')
            self.command.get_test_cases.assert_called_once_with('foobar')
            self.assertTrue(self.command.info.called)
            self.assertTrue(self.command.error.called) # out file does not exist

            # check if no label was given
            with patch('porunga.commands.test.curdir') as curdirmock:
                curdirmock.return_value = '/foo/bar'
                self.command.handle_label(None, self.command.namespace)
                self.assertEqual(self.command.get_binary.call_args,
                    call('/foo/bar', 'python'))

    def test_test_program(self):
        with patch('porunga.commands.test.Popen') as PopenMock:
            with tempfile.NamedTemporaryFile() as fout:
                fout.write(b'out')
                fout.flush()
                mock = PopenMock.return_value = Mock()
                mock.communicate = lambda: ('out', '')
                mock.returncode = 0
                info = self.command.test('foobar', 'foobar/foobar.out', 'fin', fout.name)

                self.assertEqual(PopenMock.call_args_list, [
                    call('cat fin | foobar/foobar.out', stdout=PIPE, stderr=PIPE,
                        shell=True),
                ])

                self.assertTrue(info['success'])
                self.assertIn('time', info)

    def test_test_program_fails(self):
        with patch('porunga.commands.test.Popen') as PopenMock:
            self.command.log = Mock()
            self.command.namespace.verbose = True
            mock = PopenMock.return_value = Mock()
            mock.communicate = lambda: ('out', 'err')
            mock.returncode = 1
            info = self.command.test('foobar', 'foobar/foobar.out', 'fin', 'fout')

            self.assertEqual(PopenMock.call_args_list, [
                call('cat fin | foobar/foobar.out', stdout=PIPE, stderr=PIPE,
                     shell=True),
            ])

            self.assertFalse(info['success'])
            self.assertIn('time', info)

    def test_logs(self):
        self.command.log = Mock()

        self.command.info('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored(' => foo', 'blue'), False))

        self.command.info_continuation('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored('foo', 'blue'), False))

        self.command.success('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored(' => foo', 'green'), False))

        self.command.success_continuation('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored('foo', 'green'), False))

        self.command.error('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored(' => foo', 'red'), False))

        self.command.error_continuation('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored('foo', 'red'), False))

    def test_curdir(self):
        with patch('porunga.commands.test.os') as osmock:
            osmock.path.curdir = '/foo/bar/baz'
            self.assertEqual(curdir(), '/foo/bar/baz')




import os
from porunga.utils.compat import unittest


def collector():
    start_dir = os.path.abspath(os.path.dirname(__file__))
    return unittest.defaultTestLoader.discover(start_dir)

def main():
    unittest.main()

if __name__ == '__main__':
    main()




from __future__ import print_function
import datetime
import os
import subprocess
import tempfile
import threading
import time


class TimeoutExceeded(RuntimeError):
    pass


class Command(object):

    def __init__(self, cmd, stream=None, shell=False, timeout=None):
        self.cmd = cmd
        self.process = None
        self.thread = None
        self.returncode = None
        self.stream = stream or tempfile.NamedTemporaryFile()
        self.shell = shell
        self.timeout = timeout
        self.output = ''

    def __str__(self):
        return '<Command: %s | %r>' % (self.returncode, self.cmd)

    def run_in_thread(self):
        def target():
            self.process = subprocess.Popen(
                self.cmd,
                stdout=self.stream,
                stderr=subprocess.STDOUT,
                shell=self.shell,
            )
            self.process.communicate()
            self.returncode = self.process.returncode

        self.thread = threading.Thread(target=target)
        self.thread.start()
        return self.thread

    def iter_output_for_stream(self, stream, pause=0.05):
        """
        """
        afile = open(stream.name)
        thread = self.run_in_thread()
        start = datetime.datetime.now()

        def should_run():
            should = self.returncode is None
            if self.timeout:
                delta = (datetime.datetime.now() - start).total_seconds()
                should = should and delta < self.timeout
            return should

        # we need to run at least once as at this time as for very fast commands
        # process might already finish and returncode already set
        while True:
            if afile.tell() < os.path.getsize(afile.name):
                chunk = afile.read()
                self.output += chunk
                yield chunk
            if not should_run():
                break
            time.sleep(pause)

        if thread.is_alive():
            self.process.terminate()
            thread.join()
            raise TimeoutExceeded

    def iter_output(self, pause=0.05):
        """
        Returns iterator of chunked output.

        :param cmd: command that would be passed to ``subprocess.Popen``
        :param shell: Tells if process should be run within a shell. Default: False
        :param timeout: If command exceeds given ``timeout`` in seconds,
        ``TimeoutExceeded`` exception would be raised. Default: None
        :param pause: How long (in seconds) we should wait for the output.
        Default: 0.05

        Example::

            >>> command = Command('ls -l', shell=True)
            >>> for chunk in command.iter_output():
            ...     print(chunk, end='')
        """
        with self.stream as temp:
            for chunk in self.iter_output_for_stream(temp, pause=pause):
                yield chunk

    def run(self):
        for chunk in self.iter_output():
            pass # TimeoutExceeded might be thrown here
        return self.returncode


if __name__ == '__main__':
    cmd = 'for x in `seq 3`; do echo "Doing $x"; sleep 1; done'
    #cmd = 'echo foobar'
    timeout = 0.5
    command = Command(cmd, shell=True, timeout=timeout)
    try:
        for chunk in command.iter_output(pause=0.2):
            print(chunk, end='')
    except TimeoutExceeded as err:
        print(" -> Timeouted after %s" % timeout)
    print(" -> Command finished with returncode: %s" % command.returncode)

